module Wrapybara
	class Option
		include Element
		
		attr_accessor :parent, :element
		
		def initialize(select, option)
			@option = option
			@parent = select
			xpath = option.blank? ? ".//option[normalize-space()='']" : XPath::HTML.option(option)
			@element = select.element.find(xpath) rescue nil
		end

		def should_exist
			super "Expected select #{self.parent_identifier} to have option '#{@option}'"
		end

		def should_not_exist
			super "Did not expect select #{self.parent_identifier} to have option '#{@option}'"
		end
		
		def select
			self.should_exist
			# Capybara method
			@element.select_option
		end
		
		def deselect
			self.should_exist
			# Capybara method
			@element.unselect_option
		end
		
		def selected?
			self.should_exist
			# Capybara method
			@element.selected? || @parent.element.value == @element.value
		end
		
		def should_be_selected
			raise UnmetExpectation, "Expected select #{self.parent_identifier} to have option '#{@option}' selected" unless self.selected?
		end
		
		def should_not_be_selected
			raise UnmetExpectation, "Did not expect select #{self.parent_identifier} to have option '#{@option}' selected" if self.selected?
		end
	end
end