module Wrapybara
	class Select
		include Element
		
		attr_reader :element, :identifier, :how, :scope
		
		def initialize(identifier, scope = default_scope, how = default_how)
			@identifier = identifier
			@how = how
			@scope = scope
			@element = get_element(XPath::HTML.select(identifier), scope)
		end
		
		def should_exist
			super "Expected a select #{self.element_identifier} to exist"
		end

		def should_not_exist
			super "Did not expect a select #{self.element_identifier}' to exist"
		end

		def option(option)
			Option.new(self, option)
		end

		def has_option?(option)
			self.should_exist
			self.option(option).exists?
		end

		def should_have_options(options)
			self.should_exist
			options.each do |option|
				self.option(option).should_exist
			end
		end

		def should_not_have_options(options)
			self.should_exist
			options.each do |option|
				self.option(option).should_not_exist
			end
		end

		def select(option)
			self.should_have_options([option])
			self.option(option).select
		end
		
		alias_method :choose, :select
		
		def deselect(option)
			self.should_have_options([option])
			self.option(option).deselect
		end

		def should_be_selected(option)
			self.should_have_options([option])
			self.option(option).should_be_selected
		end

		def should_not_be_selected(option)
			self.should_have_options([option])
			self.option(option).should_not_be_selected
		end
	end
end