module Wrapybara
	class Table
		include Element

		attr_reader :element, :identifier, :how, :scope, :head, :body
		
		def initialize(identifier, scope = default_scope, how = default_how)
			@identifier = identifier
			@how = how
			@scope = scope
			xpath = XPath::HTML.table(identifier)
			@element = get_element(xpath, scope)
			@head = TableHead.new(self)
			@body = TableBody.new(self)
		end
		
		def should_exist
			super "Expected a table #{self.element_identifier}' to exist"
		end

		def should_not_exist
			super "Did not expect a table #{self.element_identifier} to exist"
		end

		def should_have_columns(columns)
			self.head.should_exist
			columns.each do |column|
				raise UnmetExpectation, "Expected table #{self.element_identifier} to have column '#{column}'" unless self.has_column?(column)
			end
		end

		def should_not_have_columns(columns)
			self.head.should_exist
			columns.each do |column|
				raise UnmetExpectation, "Did not expect table #{self.element_identifier} to have column '#{column}'" if self.has_column?(column)
			end
		end
		
		def click_column(column)
			self.should_exist
			self.should_have_columns([column])
			cell = self.head.cell(column)
			link = cell.element.find('a[1]')
			link.click
		end
		
		def has_column?(label)
			@head.has_column?(label)
		end
	end # class Table
end # module Wrapybara