module Wrapybara
	class TableCell
		include Element
		
		attr_reader :parent, :element

		def initialize(parent, *args)
			args = args.first if args.first.is_a?(Array)
			
			@parent = parent
			t_what = parent.is_a?(TableHead) ? 'th' : 'td'
			xpath =
				if args.first.to_i == 0
					@content = args.first
					"*/#{t_what}[contains(normalize-space(), '#{@content}')]"
				else
					@row = args.first.to_i
					@column = args.last.to_i
					"tr[#{@row}]/#{t_what}[#{@column}]"
				end

			@element = parent.element.find(xpath) rescue nil
		end
		
		def should_exist
			super "Expected #{self.exists_message}"
		end

		def should_not_exist
			super "Did not expect #{self.exists_message}"
		end

		def should_have_content(content)
			self.should_exist
			raise UnmetExpectation, "Expected #{self.has_content_message(content)}" unless self.has_content?(content)
		end

		def should_not_content(content)
			self.should_exist
			raise UnmetExpectation, "Did not expect #{self.has_content_message(content)}" if self.has_content?(content)
		end

		def should_contain(element, identifier, how)
			self.should_exist
			raise UnmetExpectation, "Expected #{self.contains_message(element, identifier, how)}" unless self.contains?(element, identifier, how)
		end
		
		def should_not_contain(element, identifier, how)
			self.should_exist
			raise UnmetExpectation, "Did not expect #{self.contains_message(element, identifier, how)}" if self.contains?(element, identifier, how)
		end
		
		def contains?(element, identifier, how)
			child = "Wrapybara::#{element.camelize}".constantize.new(identifier, self.element.path, how)
			child.exists?
		end
		
		def exists?
			!@element.nil?
		end
		
		def has_content?(content)
			@element.text =~ /#{content}/
		end
		
		def exists_message
			parent = @parent.is_a?(TableHead) ? 'head' : 'body'
			message = "#{parent} of table #{@parent.parent.how} '#{@parent.parent.identifier}'#{within(@parent.parent.scope)} to have a cell "
			message += @content ? "with content '#{@content}'" : "at row #{@row} column #{@column}"
			message
		end
		
		def has_content_message(content)
			parent = @parent.is_a?(TableHead) ? 'head' : 'body'
			message = "#{parent} cell at row #{@row} column #{@column} of table #{@parent.parent.how} '#{@parent.parent.identifier}'#{within(@parent.parent.scope)}"
			message += " to have content '#{content}'"
			message
		end
		
		def contains_message(element, identifier, how)
			parent = @parent.is_a?(TableHead) ? 'head' : 'body'
			message = "#{parent} cell at row #{@row} column #{@column} of table #{@parent.parent.how} '#{@parent.parent.identifier}'#{within(@parent.parent.scope)}"
			message += " to contain a[n] #{element} #{how} '#{identifier}'"
			message
		end
	end
end