Given /^(?:I )?debug$/ do
	require 'ruby-debug'
	debugger
	puts "\n\nDEBUGGER\n\n"
end

Given /^(?:I )?reload(?: the page)?$/ do
	visit current_path
end

Given /^(?:I )?display "([^"]+)"$/ do |something|
	puts "\n\n#{something}\n\n"
end

Given /^(?:I )?wait(?: (?:for )?(\d+) seconds)?$/ do |seconds|
	seconds ||= '2'
	sleep(seconds.to_i)
end

Given /^there (should|should not) be a[n]? (.+) (labeled|identified by|for|titled) "([^"]+)"(?: (?:in|within) "([^"]+)")?$/ do |expectation, element, how, identifier, scope|
	element.gsub!(' ', '_')
	expectation.gsub!(' ', '_')
	"Wrapybara::#{element.camelize}".constantize.new(identifier, scope, how).send("#{expectation}_exist")
end

Given /^the table (labeled|identified by) "([^"]+)"(?: (?:in|within) "([^"]+)")? (should|should not) have column[s]?(?: "([^"]+)")?$/ do |how, identifier, scope, expectation, column, table|
	columns = column ? [column] : (table.hashes.collect { |row| row['label'] || '' })
	Wrapybara::Table.new(identifier, scope, how).send("#{expectation}_have_columns", columns)
end

Given /^the select (labeled|identified by) "([^"]+)"(?: (?:in|within) "([^"]+)")? (should|should not) have option[s]?(?: "([^"]+)")?(?: (selected))?$/ do |how, identifier, scope, expectation, option, selected, table|
	options = option ? [option] : (table.hashes.collect { |row| row['label'] || '' })
	expectation.gsub!(' ', '_')
	if selected
		options.each do |option|
			Wrapybara::Select.new(identifier, scope, how).option(option).send("#{expectation}_be_selected")
		end
	else
		Wrapybara::Select.new(identifier, scope, how).send("#{expectation}_have_options", options)
	end
end

# In this step, the label capture must use an * to accept empty strings, ie, selecting the blank option typically at the top of the options
Given /^(?:I )?(choose|select|deselect) "([^"]*)" from the select (labeled|identified by) "([^"]+)"?(?: (?:in|within) "([^"]+)")?$/ do |action, option, how, identifier, scope|
	Wrapybara::Select.new(identifier, scope, how).send(action.to_sym, option)
end

Given /^the select (labeled|identified by) "([^"]+)"?(?: (?:in|within) "([^"]+)")? (should|should not) have "([^"]*)" selected$/ do |how, identifier, scope, expectation, option|
	expectation.gsub!(' ', '_')
	Wrapybara::Select.new(identifier, scope, how).option(option).send("#{expectation}_be_selected")
end

Given /^the (.+) (labeled|identified by|titled) "([^"]+)"(?: (?:in|within) "([^"]+)")? (should|should not) be (.+)$/ do |element, how, identifier, scope, expectation, state|
	element.gsub!(' ', '_')
	expectation.gsub!(' ', '_')
	"Wrapybara::#{element.camelize}".constantize.new(identifier, scope, how).send("#{expectation}_be_#{state}")
end

Given /^the (.+) (labeled|identified by|titled) "([^"]+)"(?: (?:in|within) "([^"]+)")? (should|should not) have attribute "([^"]+)" with value "([^"]+)"$/ do |element, how, identifier, scope, expectation, attribute, value|
	element.gsub!(' ', '_')
	expectation.gsub!(' ', '_')
	"Wrapybara::#{element.camelize}".constantize.new(identifier, scope, how).send("#{expectation}_have_attribute", attribute, value)
end

Given /^(?:I )?(check|uncheck) the (checkbox|radio button) (labeled|identified by) "([^"]+)"(?: (?:in|within) "([^"]+)")?$/ do |action, element, how, identifier, scope|
	element.gsub!(' ', '_')
	"Wrapybara::#{element.camelize}".constantize.new(identifier, scope, how).send(action)
end

# In this step, the label capture must use an * to accept empty strings, so we can clear the field
Given /^(?:I )?fill in the (text field|password field|text area) (labeled|identified by) "([^"]+)"(?: (?:in|within) "([^"]+)")? with "([^"]*)"$/ do |element, how, identifier, scope, content|
	content ||= ''
	element.gsub!(' ', '_')
	"Wrapybara::#{element.camelize}".constantize.new(identifier, scope, how).fill_in(content)
end

Given /^(?:I )?click the (button|link) (labeled|identified by|titled) "([^"]+)"(?: (?:in|within) "([^"]+)")?$/ do |element, how, identifier, scope|
	"Wrapybara::#{element.capitalize}".constantize.new(identifier, scope, how).click
end

# for finding unlabeled links
Given /^there (should|should not) be a link to "([^"]+)"(?: (?:in|within) "([^"]+)")?$/ do |expectation, destination, scope|
	expectation.gsub!(' ', '_')
	Wrapybara::Link.new(destination, scope, 'to').send("#{expectation}_exist")
end

Given /^there (should|should not) be a link to the "([^"]+)" page(?: (?:in|within) "([^"]+)")?$/ do |expectation, destination, scope|
	expectation.gsub!(' ', '_')
	destination = path_to(destination)
	Wrapybara::Link.new(destination, scope, 'to').send("#{expectation}_exist")
end

# for clicking unlabeled links
Given /^(?:I )?click the link to the "([^"]+)" page(?: (?:in|within) "([^"]+)")?$/ do |destination, scope|
	destination = path_to(destination)
	Wrapybara::Link.new(destination, scope, 'to').click
end

# for clicking table column headers (as in sortable columns)
Given /^(?:I )?click column "([^"]+)" of the table (labeled|identified by) "([^"]+)"(?: (?:in|within) "([^"]+)")?$/ do |column, how, table, scope|
	Wrapybara::Table.new(table, scope, how).click_column(column)
end

# content
Given /^(?:I )?(should|should not) see "([^"]+)"(?: within "([^"]+)")?$/ do |expectation, content, scope|
	expectation.gsub!(' ', '_')
	Wrapybara::Content.new(content, scope).send("#{expectation}_exist")
end

Given /^"([^"]+)"(?: within "([^"]+)")? (should|should not) be visible$/ do |content, scope, expectation|
	expectation.gsub!(' ', '_')
	Wrapybara::Content.new(content, scope).send("#{expectation}_be_visible")
end

# for clicking things like headers that have expandable detail below them
Given /^I click the text "([^"]+)"(?: within "([^"]+)")?$/ do |content, scope|
	Wrapybara::Content.new(content, scope).click
end

# fillable fields (text field, password field, text area)
Given /^the (.+) (labeled|identified by) "([^"]+)"(?: (?:in|within) "([^"]+)")? (should|should not) have content "([^"]+)"$/ do |element, how, identifier, scope, expectation, content|
	element.gsub!(' ', '_')
	expectation.gsub!(' ', '_')
	"Wrapybara::#{element.camelize}".constantize.new(identifier, scope, how).send("#{expectation}_have_content", content)
end

# table content
Given /^(head|body)?\s?row (\d+) column (\d+) of the table (labeled|identified by) "([^"]+)"(?: (?:in|within) "([^"]+)")? (should|should not) have content "([^"]+)"$/ do |element, row, column, how, identifier, scope, expectation, content|
	element ||= 'body'
	expectation.gsub!(' ', '_')
	Wrapybara::Table.new(identifier, scope, how).send(element).cell(row, column).send("#{expectation}_have_content", content)
end

# other elements in table cells
Given /^(head|body)?\s?row (\d+) column (\d+) of the table (labeled|identified by) "([^"]+)"(?: (?:in|within) "([^"]+)")? (should|should not) contain a[n]? (.+) (labeled|identified by) "([^"]+)"$/ do |element, row, column, how, identifier, scope, expectation, child_element, child_how, child_identifier|
	element ||= 'body'
	expectation.gsub!(' ', '_')
	Wrapybara::Table.new(identifier, scope, how).send(element).cell(row, column).send("#{expectation}_contain", child_element, child_identifier, child_how)
end
